<?php
/*
Plugin Name: TagCloud widget
Description: Adds a tag cloud sidebar widget
Author: Anders Dahnielson
Version: 1.2
Author URI: http://dahnielson.com
*/

/*  Copyright 2006  Anders Dahnielson (email : anders@dahnielson)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

load_plugin_textdomain('tagcloud', 'wp-content/plugins/widgets');

function widget_tagcloud_init()
{
	if ( !function_exists('register_sidebar_widget') )
		return;

	function widget_tagcloud($args) 
	{
		extract($args);

		$options = get_option('widget_tagcloud');
		$title = $options['title'];
		$child_of = $options['child_of'];
		
		global $wpdb;
		$categories = $wpdb->get_results("SELECT cat.cat_ID, cat.cat_name, cat.category_count,
MAX(stat.category_count) AS max, MIN(stat.category_count) AS min,
MAX(stat.category_count) - MIN(stat.category_count) / 3 AS distribution
FROM $wpdb->categories AS stat, $wpdb->categories AS cat
WHERE stat.category_parent = '$child_of' AND stat.category_count > 0 AND cat.category_parent = '$child_of' AND cat.category_count > 0 
GROUP BY cat.cat_name");
	
		$thelist = '';
		foreach ($categories as $category)
			{
				if ($category->category_count == $category->min)
					$class = "tag-smallest";
				elseif ($category->category_count == $category->max)
					$class = "tag-largest";
				elseif ($category->category_count > ($category->min + ($category->distribution*2)))
					$class = "tag-large";
				elseif ($category->category_count > ($category->min + $category->distribution))
					$class = "tag-medium";
				else
					$class = "tag-small";

				if ($category->category_count == 1)
					$thelist .= '<li><a href="'.get_category_link($category->cat_ID).'" title="'.sprintf( __('%1$d post tagged as %2$s', 'tagcloud'), $category->category_count, $category->cat_name ).'" rel="tag" class="'.$class.'">'.$category->cat_name.'</a> <span class="tag-freq">('.intval($category->category_count).')</span></li>';
				elseif ($category->category_count > 1)
					$thelist .= '<li><a href="'.get_category_link($category->cat_ID).'" title="'.sprintf( __('%1$d posts tagged as %2$s', 'tagcloud'), $category->category_count, $category->cat_name ).'" rel="tag" class="'.$class.'">'.$category->cat_name.'</a> <span class="tag-freq">('.intval($category->category_count).')</span></li>';
			}

		echo $before_widget . $before_title . $title . $after_title;
		echo "<ul class=\"tag-cloud\">" . $thelist . "</ul>";
		echo $after_widget;
	}
	
	function widget_tagcloud_control()
	{
		$options = get_option('widget_tagcloud');
		if ( !is_array($options) )
			$options = array('title'=>'Topics', 'child_of'=>'0');
		if ( $_POST['tagcloud-submit'] ) {
			$options['title'] = strip_tags(stripslashes($_POST['tagcloud-title']));
			$options['child_of'] = strip_tags(stripslashes($_POST['tagcloud-child_of']));
			update_option('widget_tagcloud', $options);
		}

		$title = htmlspecialchars($options['title'], ENT_QUOTES);
		$child_of = htmlspecialchars($options['child_of'], ENT_QUOTES);

		echo '<p style="text-align:right;"><label for="tagcloud-title">' . __('Title:') . ' <input style="width: 200px;" id="tagcloud-title" name="tagcloud-title" type="text" value="'.$title.'" /></label></p>';
		echo '<p style="text-align:right;"><label for="tagcloud-child_of">' . __('Child of:') . ' <input style="width: 200px;" id="tagcloud-child_of" name="tagcloud-child_of" type="text" value="'.$child_of.'" /></label></p>';
		echo '<input type="hidden" id="tagcloud-submit" name="tagcloud-submit" value="1" />';
	}
	
	register_sidebar_widget(array('TagCloud', 'widgets'), 'widget_tagcloud');
	register_widget_control(array('TagCloud', 'widgets'), 'widget_tagcloud_control', 300, 100);
}

add_action('widgets_init', 'widget_tagcloud_init');

?>
